"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.genFormCodeSnippet = exports.genFetchCode = exports.genFormImportResources = void 0;
const toycode_core_1 = require("@qlin/toycode-core");
const lodash_1 = require("lodash");
const type_1 = require("../type");
const componentMap_1 = require("../componentMap");
const shared_1 = require("./shared");
const rules_1 = require("./rules");
const utils_1 = require("../utils");
function genFormImportResources(ctx, fields) {
    const importSources = [
        {
            imported: 'FForm',
            type: toycode_core_1.ImportType.ImportSpecifier,
            source: '@fesjs/fes-design',
        },
        {
            imported: 'FFormItem',
            type: toycode_core_1.ImportType.ImportSpecifier,
            source: '@fesjs/fes-design',
        },
    ];
    fields.forEach((field) => {
        const comp = (0, componentMap_1.componentMap)(field.component.componentName);
        importSources.push({
            imported: comp.name,
            type: toycode_core_1.ImportType.ImportSpecifier,
            source: '@fesjs/fes-design',
        });
        if (comp.name === 'FSelect' && field.options?.length) {
            importSources.push({
                imported: comp.subName,
                type: toycode_core_1.ImportType.ImportSpecifier,
                source: '@fesjs/fes-design',
            });
        }
        else if (comp.name !== 'FSelect' && comp.subName) {
            importSources.push({
                imported: comp.subName,
                type: toycode_core_1.ImportType.ImportSpecifier,
                source: '@fesjs/fes-design',
            });
        }
    });
    importSources.push(...(0, shared_1.genImportedMappingCode)(ctx, fields));
    return importSources;
}
exports.genFormImportResources = genFormImportResources;
function genFetchCode(ctx, fields) {
    let fetchCode = '';
    const importSources = [];
    fields.forEach((field) => {
        if (field.apiSchema) {
            const relateParams = fields.filter((item) => {
                return (field.apiSchema.params || []).find((param) => param.name === item.name);
            });
            if (!relateParams.length) {
                importSources.push({
                    imported: 'useFetch',
                    type: toycode_core_1.ImportType.ImportSpecifier,
                    source: ctx.getUseFetchImp(),
                });
                fetchCode += `
                const {data: ${(0, utils_1.genOptionsName)(field.name)} } = useFetch('${field.apiSchema.url}', {
                    defaultValue: () => [],
                    ${field.apiSchema.resData.pick?.length
                    ? `pick: ${JSON.stringify(field.apiSchema.resData.pick)},`
                    : ''}
                });
            `;
            }
            else {
                importSources.push({
                    imported: 'request',
                    type: toycode_core_1.ImportType.ImportSpecifier,
                    source: '@fesjs/fes',
                });
                importSources.push({
                    imported: 'ref',
                    type: toycode_core_1.ImportType.ImportSpecifier,
                    source: 'vue',
                });
                importSources.push({
                    imported: 'watch',
                    type: toycode_core_1.ImportType.ImportSpecifier,
                    source: 'vue',
                });
                const relateParamsValues = relateParams.map((item) => {
                    return `searchParams.${item.name}`;
                });
                fetchCode += `
                const ${(0, utils_1.genOptionsName)(field.name)} = ref([]);
                watch(() => [${relateParamsValues.join(', ')}], async () => {
                    searchParams.${field.name} = null;
                    ${(0, utils_1.genOptionsName)(field.name)}.value = [];
                    if (${relateParamsValues.join(' != null && ')} != null) {
                        const result = await request('${field.apiSchema.url}', {
                            ${relateParamsValues
                    .map((item) => {
                    return `${item.split('.')[1]}: ${item}`;
                })
                    .join(',')}
                        });
                        ${(0, utils_1.genOptionsName)(field.name)}.value = ${field.apiSchema.resData.pick
                    ? `result.${field.apiSchema.resData.pick[0]}`
                    : 'result'}
                        
                    }
                }, {
                    immediate: true
                });
                `;
            }
        }
    });
    return {
        content: fetchCode,
        importSources,
    };
}
exports.genFetchCode = genFetchCode;
function genSetupCode(ctx, apiSchema) {
    const { importSources, content } = genFetchCode(ctx, apiSchema.params);
    return {
        importSources: [
            ...genFormImportResources(ctx, apiSchema.params),
            ...importSources,
        ],
        content: `
        const formRules = {
            ${genRules(apiSchema.params)}
        };
        ${content}
        `,
    };
}
function genRules(fields) {
    return fields
        .filter((field) => {
        return field.component.required || !(0, lodash_1.isEmpty)(field.component.rules);
    })
        .map((field) => {
        const rules = field.component.rules || [];
        if (field.component.required &&
            !rules.find((rule) => rule.type === 'required')) {
            rules.push({
                type: type_1.RuleTypeEnum.required,
            });
        }
        return `
        ${field.name}: [
           ${rules
            .map((rule) => {
            return rules_1.rulesHandler[rule.type](rule, field.type);
        })
            .join(',')} 
        ]
        `;
    })
        .join(',');
}
function genFormItems(fields) {
    return fields.map((field) => {
        const comp = (0, componentMap_1.componentMap)(field.component.componentName);
        const renderCompProps = {
            ...field.component.props,
            ...comp.props,
        };
        let children = null;
        if (field.mappingId) {
            renderCompProps.options = {
                type: toycode_core_1.ExtensionType.JSExpression,
                value: field.mappingId,
            };
        }
        else if (field.options?.length) {
            children = (0, shared_1.handleComponentOptions)(field.options, comp.subName);
        }
        else if (field.apiSchema) {
            renderCompProps.options = {
                type: toycode_core_1.ExtensionType.JSExpression,
                value: `${(0, utils_1.genOptionsName)(field.name)}`,
            };
        }
        return {
            componentName: 'FFormItem',
            props: {
                label: field.title,
                prop: field.name,
            },
            children: [
                {
                    componentName: comp.name,
                    props: renderCompProps,
                    directives: {
                        'v-model': `formModel.${field.name}`,
                    },
                    children,
                },
            ],
        };
    });
}
function genFormCodeSnippet(ctx, apiSchema, parentId) {
    return {
        setup: genSetupCode(ctx, apiSchema),
        component: {
            parentId,
            componentName: 'FForm',
            props: {
                ref: 'formRefEl',
                rules: {
                    value: 'formRules',
                    type: toycode_core_1.ExtensionType.JSExpression,
                },
                model: {
                    value: 'formModel',
                    type: toycode_core_1.ExtensionType.JSExpression,
                },
                labelWidth: (0, shared_1.genLabelWidth)(apiSchema.params),
            },
            children: genFormItems(apiSchema.params),
        },
    };
}
exports.genFormCodeSnippet = genFormCodeSnippet;
