"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.genRelationModals = exports.genViewModal = exports.genUpdateModal = exports.genAddModal = void 0;
const toycode_core_1 = require("@qlin/toycode-core");
const lodash_1 = require("lodash");
const utils_1 = require("../utils");
const constants_1 = require("../constants");
const shared_1 = require("./shared");
const form_1 = require("./form");
const descriptions_1 = require("./descriptions");
const table_1 = require("./table");
const getCommonModalImportSource = (ctx) => {
    return [
        {
            imported: 'FModal',
            type: toycode_core_1.ImportType.ImportSpecifier,
            source: '@fesjs/fes-design',
        },
        {
            imported: 'FMessage',
            type: toycode_core_1.ImportType.ImportSpecifier,
            source: '@fesjs/fes-design',
        },
        {
            imported: 'useModal',
            type: toycode_core_1.ImportType.ImportSpecifier,
            source: ctx.getUseModalImp(),
        },
        {
            imported: 'request',
            type: toycode_core_1.ImportType.ImportSpecifier,
            source: '@fesjs/fes',
        },
    ];
};
function getModalComponent(title) {
    return {
        id: (0, toycode_core_1.genComponentId)(),
        componentName: 'FModal',
        props: {
            title,
            maskClosable: false,
            top: 54,
            displayDirective: 'if',
        },
        events: {
            ok: 'confirm',
        },
        directives: {
            'v-model:show': 'innerVisible',
        },
        children: [],
    };
}
function genAddModalCodeSnippet(ctx, modal) {
    return {
        setup: {
            importSources: getCommonModalImportSource(ctx),
            content: `
            const onConfirm = async (data) => {
                const res = await request('${modal.apiSchema.url}', data);
                FMessage.success('新增成功');
                props.onSuccess(data, res);
            };
            const { formRefEl, formModel, confirm, innerVisible } = useModal({ props, emit, initData, onConfirm });
            `,
        },
        component: getModalComponent(modal.meta.title || '新增'),
    };
}
function genViewModalCodeSnippet(ctx, modal) {
    return {
        setup: {
            importSources: [
                {
                    imported: 'FModal',
                    type: toycode_core_1.ImportType.ImportSpecifier,
                    source: '@fesjs/fes-design',
                },
                {
                    imported: 'FButton',
                    type: toycode_core_1.ImportType.ImportSpecifier,
                    source: '@fesjs/fes-design',
                },
                {
                    imported: 'useNormalModel',
                    type: toycode_core_1.ImportType.ImportSpecifier,
                    source: ctx.getUseModelImp(),
                },
            ],
            content: `
            const [innerVisible, updateVisible]= useNormalModel(props, emit, {
                prop: 'visible',
            });
            `,
        },
        component: {
            id: (0, toycode_core_1.genComponentId)(),
            componentName: 'FModal',
            props: {
                title: modal.meta.title || '查看',
                maskClosable: false,
                displayDirective: 'if',
                top: 54,
            },
            directives: {
                'v-model:show': 'innerVisible',
            },
            children: [],
            slots: [
                {
                    name: 'footer',
                    component: {
                        componentName: 'FButton',
                        props: {
                            type: 'primary',
                        },
                        events: {
                            click: 'updateVisible(false)',
                        },
                        children: ['关闭'],
                    },
                },
            ],
        },
    };
}
function genInitData(addModal) {
    const fieldDefaultValue = addModal.apiSchema.params.map((item) => {
        return `${item.name}: ${(0, shared_1.getDefaultValue)(item)}`;
    });
    return {
        importSources: [],
        content: `
        const initData = {
            ${fieldDefaultValue.join(',')}
        };
        `,
    };
}
function genAddModal(ctx, addModal, modalDir) {
    const modalSnippet = genAddModalCodeSnippet(ctx, addModal);
    const formSnippet = (0, form_1.genFormCodeSnippet)(ctx, addModal.apiSchema, modalSnippet.component.id);
    const codeSnippets = [modalSnippet, formSnippet];
    const { children, setupCodes } = (0, shared_1.mergeCodeSnippets)(codeSnippets);
    return {
        componentName: 'SFCComponent',
        dir: modalDir,
        fileName: 'addModal.vue',
        children,
        setupCodes: [genInitData(addModal), ...setupCodes],
        propsDefinition: [
            {
                name: 'visible',
                propType: 'Boolean',
            },
            {
                name: 'onSuccess',
                propType: 'Function',
            },
        ],
        emitsDefinition: ['update:visible'],
    };
}
exports.genAddModal = genAddModal;
function genUpdateModalCodeSnippet(ctx, modal) {
    return {
        setup: {
            importSources: [
                ...getCommonModalImportSource(ctx),
                {
                    imported: 'computed',
                    type: toycode_core_1.ImportType.ImportSpecifier,
                    source: 'vue',
                },
            ],
            content: `
            const onConfirm = async (data) => {
                const res = await request('${modal.apiSchema.url}', data);
                FMessage.success('修改成功');
                props.onSuccess(data, res);
            };
            const { formRefEl, formModel, confirm, innerVisible } = useModal({ props, emit, initData: computed(() => props.${constants_1.ROW_DATA_PROP_NAME}), onConfirm });
            `,
        },
        component: getModalComponent(modal.meta.title || '编辑'),
    };
}
function genViewTableSnippet(apiSchema, parentId) {
    const fileName = (0, utils_1.genFileNameByPath)(apiSchema.url);
    const componentName = (0, utils_1.genComponentName)(fileName);
    return {
        setup: {
            importSources: [
                {
                    imported: componentName,
                    type: toycode_core_1.ImportType.ImportDefaultSpecifier,
                    source: `./${fileName}.vue`,
                },
            ],
            content: '',
        },
        component: {
            parentId,
            componentName,
            props: {
                [constants_1.ROW_DATA_PROP_NAME]: {
                    type: toycode_core_1.ExtensionType.JSExpression,
                    value: constants_1.ROW_DATA_PROP_NAME,
                },
            },
        },
    };
}
function genUpdateModal(ctx, modal, modalDir) {
    const modalSnippet = genUpdateModalCodeSnippet(ctx, modal);
    const formSnippet = (0, form_1.genFormCodeSnippet)(ctx, modal.apiSchema, modalSnippet.component.id);
    const codeSnippets = [modalSnippet, formSnippet];
    if (!(0, lodash_1.isEmpty)(modal.viewProps)) {
        codeSnippets.push((0, descriptions_1.genDescriptionsSnippet)(modal.viewProps, modalSnippet.component.id));
    }
    if (!(0, lodash_1.isEmpty)(modal.viewExtraData)) {
        codeSnippets.push(genViewTableSnippet(modal.viewExtraData, modalSnippet.component.id));
    }
    const { children, setupCodes } = (0, shared_1.mergeCodeSnippets)(codeSnippets);
    const result = [
        {
            componentName: 'SFCComponent',
            dir: modalDir,
            fileName: 'updateModal.vue',
            children,
            setupCodes: [...setupCodes],
            propsDefinition: [
                {
                    name: 'visible',
                    propType: 'Boolean',
                },
                {
                    name: constants_1.ROW_DATA_PROP_NAME,
                    propType: 'Object',
                },
                {
                    name: 'onSuccess',
                    propType: 'Function',
                },
            ],
            emitsDefinition: ['update:visible'],
        },
    ];
    if (modal.viewExtraData) {
        result.push((0, table_1.genViewTableComponent)(ctx, modal.viewExtraData, modalDir));
    }
    return result;
}
exports.genUpdateModal = genUpdateModal;
function genViewModal(ctx, modal, modalDir) {
    const modalSnippet = genViewModalCodeSnippet(ctx, modal);
    const codeSnippets = [modalSnippet];
    if (!(0, lodash_1.isEmpty)(modal.viewProps)) {
        codeSnippets.push((0, descriptions_1.genDescriptionsSnippet)(modal.viewProps, modalSnippet.component.id));
    }
    if (!(0, lodash_1.isEmpty)(modal.viewExtraData)) {
        codeSnippets.push(genViewTableSnippet(modal.viewExtraData, modalSnippet.component.id));
    }
    const { children, setupCodes } = (0, shared_1.mergeCodeSnippets)(codeSnippets);
    const result = [
        {
            componentName: 'SFCComponent',
            dir: modalDir,
            fileName: 'viewModal.vue',
            children,
            setupCodes: [...setupCodes],
            propsDefinition: [
                {
                    name: 'visible',
                    propType: 'Boolean',
                },
                {
                    name: constants_1.ROW_DATA_PROP_NAME,
                    propType: 'Object',
                },
            ],
            emitsDefinition: ['update:visible'],
        },
    ];
    if (modal.viewExtraData) {
        result.push((0, table_1.genViewTableComponent)(ctx, modal.viewExtraData, modalDir));
    }
    return result;
}
exports.genViewModal = genViewModal;
function formatModalApiSchema(modal) {
    if (modal.apiSchema?.params) {
        modal.apiSchema.params = modal.apiSchema.params.filter((item) => item.checked);
    }
}
function genRelationModals(ctx, pageConfig) {
    const modals = [];
    if ((0, utils_1.hasModal)(pageConfig)) {
        const modalDir = (0, utils_1.genModalDir)(ctx, pageConfig);
        pageConfig.relationModals.forEach((modal) => {
            formatModalApiSchema(modal);
            if (modal.type === 'add') {
                modals.push(genAddModal(ctx, modal, modalDir));
            }
            else if (modal.type === 'update') {
                modals.push(...genUpdateModal(ctx, modal, modalDir));
            }
            else if (modal.type === 'view') {
                modals.push(...genViewModal(ctx, modal, modalDir));
            }
        });
    }
    return modals;
}
exports.genRelationModals = genRelationModals;
