"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.genViewTableComponent = exports.genTableTemplate = exports.genTableSetupCode = void 0;
const toycode_core_1 = require("@qlin/toycode-core");
const utils_1 = require("../utils");
const shared_1 = require("./shared");
const constants_1 = require("../constants");
const config_1 = require("../config");
function genTransform(ctx, apiSchema) {
    const importSources = [];
    const mappingCode = apiSchema.resData.fields
        .map((item) => {
        if (item.mappingId) {
            return `item.${item.alias} = getTargetLabel(${item.mappingId}, item.${item.name})`;
        }
        return null;
    })
        .filter(Boolean)
        .join(';\n');
    if (mappingCode) {
        importSources.push({
            imported: 'getTargetLabel',
            type: toycode_core_1.ImportType.ImportSpecifier,
            source: ctx.getUtilsFilePathImp(),
        }, ...(0, shared_1.genImportedMappingCode)(ctx, apiSchema.resData.fields));
    }
    const timeCode = apiSchema.resData.fields
        .map((item) => {
        if (item.name.endsWith('Time')) {
            return `item.${item.alias} = item.${item.name} ? format(item.${item.name}, 'yyyy-MM-dd HH:mm:ss') : ''`;
        }
        return null;
    })
        .filter(Boolean)
        .join(';\n');
    if (timeCode) {
        importSources.push({
            imported: 'format',
            type: toycode_core_1.ImportType.ImportSpecifier,
            source: 'date-fns',
        });
        ctx.dependence.addPackage(config_1.dateFnsDependency);
    }
    if (timeCode || mappingCode) {
        return {
            importSources: importSources,
            content: `transform(data) {
                return data.map((item) => {
                    ${mappingCode}
                    ${timeCode}
                    return item;
                })
            },`,
        };
    }
    return {
        importSources: [],
        content: '',
    };
}
// FEATURE 没想明白以后 formatParams 还有那些形式，以后遇到再对这块代码进行优化
function genFormatParams(ctx, apiSchema) {
    const targetField = apiSchema.params.find((item) => !!item.mapFields);
    if (targetField) {
        ctx.dependence.addPackage(config_1.dateFnsDependency);
        const format = targetField.component.props.type === 'daterangetime'
            ? 'yyyy-MM-dd HH:mm:ss'
            : 'yyyy-MM-dd';
        return {
            importSources: [
                {
                    imported: 'format',
                    type: toycode_core_1.ImportType.ImportSpecifier,
                    source: 'date-fns',
                },
            ],
            content: `
            formatParams(params) {
                if (params.${targetField.name}) {
                    const rawData = params.${targetField.name};
                    delete params.${targetField.name};
                    params.${targetField.mapFields[0]} = format(rawData[0], '${format}');
                    params.${targetField.mapFields[1]} = format(rawData[1], '${format}');
                }
    
                return params;
            },
            `,
        };
    }
    return {
        importSources: [],
        content: '',
    };
}
function genTableSetupCode(ctx, apiSchema, options = {}) {
    const importSources = [
        {
            imported: 'FTable',
            type: toycode_core_1.ImportType.ImportSpecifier,
            source: '@fesjs/fes-design',
        },
        {
            imported: 'FTableColumn',
            type: toycode_core_1.ImportType.ImportSpecifier,
            source: '@fesjs/fes-design',
        },
    ];
    const result = ['dataSource'];
    if (apiSchema.pagination) {
        result.push('pagination', 'changePage', 'changePageSize');
    }
    if (options.hasRefresh) {
        result.push('refresh');
    }
    const functionName = apiSchema.pagination ? 'useTable' : 'useSimpleTable';
    const dataField = (0, utils_1.getDataField)(apiSchema);
    const pageField = (0, utils_1.getPageField)(apiSchema);
    if (apiSchema.pagination) {
        importSources.push({
            imported: 'FPagination',
            type: toycode_core_1.ImportType.ImportSpecifier,
            source: '@fesjs/fes-design',
        });
    }
    importSources.push({
        imported: functionName,
        type: toycode_core_1.ImportType.ImportSpecifier,
        source: ctx.getUseTableImp(),
    });
    const transformCode = genTransform(ctx, apiSchema);
    const formatCode = genFormatParams(ctx, apiSchema);
    return {
        importSources: [
            ...importSources,
            ...transformCode.importSources,
            ...formatCode.importSources,
        ],
        content: `
        const { ${result.join(', ')} } = ${functionName}({
            api: '${apiSchema.url}',
            ${options.isInit === false ? 'isInit: false,' : ''}
            ${dataField ? `dataField: '${dataField}',` : ''}
            ${pageField ? `pageField: '${pageField}',` : ''}
            ${apiSchema.params.length ? `params: {...initSearchParams},` : ''}
            ${/post/i.test(apiSchema.method)
            ? ''
            : `method: '${apiSchema.method}',`}
            ${formatCode.content}
            ${transformCode.content}
        })
        `,
    };
}
exports.genTableSetupCode = genTableSetupCode;
function genPaginationComp() {
    const paginationComp = {
        componentName: 'FPagination',
        props: {
            class: 'common-page-pagination',
            currentPage: {
                type: toycode_core_1.ExtensionType.JSExpression,
                value: 'pagination.currentPage',
            },
            totalCount: {
                type: toycode_core_1.ExtensionType.JSExpression,
                value: 'pagination.totalCount',
            },
            pageSize: {
                type: toycode_core_1.ExtensionType.JSExpression,
                value: 'pagination.pageSize',
            },
            showSizeChanger: true,
            showTotal: true,
        },
        events: {
            change: 'changePage',
            pageSizeChange: 'changePageSize',
        },
        directives: {
            'v-if': 'pagination.totalCount > 0',
        },
    };
    return paginationComp;
}
function genTableComponent(columns) {
    const tableComp = {
        componentName: 'FTable',
        props: {
            data: {
                type: toycode_core_1.ExtensionType.JSExpression,
                value: 'dataSource',
            },
        },
        children: [],
    };
    for (const item of columns) {
        if (item.checked) {
            tableComp.children.push({
                componentName: 'FTableColumn',
                props: {
                    prop: item.alias || item.name,
                    label: item.title,
                },
            });
        }
    }
    return tableComp;
}
function genTableTemplate(apiSchema) {
    const children = [];
    children.push(genTableComponent(apiSchema.resData.fields));
    if (apiSchema.pagination) {
        children.push(genPaginationComp());
    }
    return children;
}
exports.genTableTemplate = genTableTemplate;
function genRefreshTableCode() {
    return {
        importSources: [
            {
                imported: 'watch',
                type: toycode_core_1.ImportType.ImportSpecifier,
                source: 'vue',
            },
            {
                imported: 'isEmpty',
                type: toycode_core_1.ImportType.ImportSpecifier,
                source: 'lodash-es',
            },
        ],
        content: `
        watch(() => props.${constants_1.ROW_DATA_PROP_NAME}, () => {
            if (!isEmpty(props.${constants_1.ROW_DATA_PROP_NAME})) {
                refresh(props.${constants_1.ROW_DATA_PROP_NAME});
            }
        }, {
            immediate: true
        })
        `,
    };
}
function genViewTableComponent(ctx, apiSchema, modalDir) {
    apiSchema.resData.fields = (0, shared_1.formatResData)(apiSchema.resData.fields);
    return {
        componentName: 'SFCComponent',
        dir: modalDir,
        fileName: `${(0, utils_1.genFileNameByPath)(apiSchema.url)}.vue`,
        children: genTableTemplate(apiSchema),
        setupCodes: [
            genTableSetupCode(ctx, apiSchema, {
                hasRefresh: true,
                isInit: false,
            }),
            genRefreshTableCode(),
        ],
        propsDefinition: [
            {
                name: constants_1.ROW_DATA_PROP_NAME,
                propType: 'Object',
            },
        ],
    };
}
exports.genViewTableComponent = genViewTableComponent;
